/*
    zgpio.c - GPIO library
    Copyright (C) 2018 Ladislav Vaiz <ok1zia@nagano.cz>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 2 as published by the Free Software Foundation.

*/

#include <libziaint.h>
#include <zgpio.h>
#include <zfile.h>
#include <zstr.h>

#include <glib.h>   
#include <stdlib.h>
#include <string.h>

#define SYSFS_EXPORT "/sys/class/gpio/export"
#define SYSFS_UNEXPORT "/sys/class/gpio/unexport"
#define SYSFS_DIRECTION "/sys/class/gpio/gpio%d/direction"
#define SYSFS_VALUE "/sys/class/gpio/gpio%d/value"


struct zgpio *zgpio_init_nr(int nr){
    struct zgpio *gpio = g_new0(struct zgpio, 1);
    gpio->nr = nr;
    gpio->name = g_strdup_printf("P%c%D", 'A' + (nr % 32), nr % 32);

    zfile_printfile(SYSFS_EXPORT, "%d", gpio->nr);

    return gpio;
}

struct zgpio *zgpio_init_name(const char *name){
	struct zgpio *gpio;

    if (name == NULL || strlen(name) < 3) return NULL;
    if (z_char_uc(name[0]) != 'P') return NULL;
    
    gpio = g_new0(struct zgpio, 1);
    gpio->name = g_strdup(name);
    gpio->nr = atoi(name + 2);
    gpio->nr += (z_char_uc(gpio->name[1]) - 'A') * 32;

	if (zfile_printfile(SYSFS_EXPORT, "%d", gpio->nr) < 0){
		zgpio_free(gpio);
		return NULL;
	}

    return gpio;
}

void zgpio_free(struct zgpio *gpio){
    if (!gpio) return;

    if (gpio->unexport_after) zfile_printfile(SYSFS_UNEXPORT, "%d", gpio->nr);

    g_free(gpio->name);
    g_free(gpio);
}

int zgpio_dir_input(struct zgpio *gpio){
    char file[64];

    sprintf(file, SYSFS_DIRECTION, gpio->nr);
    return zfile_printfile(file, "in");
}


int zgpio_dir_output(struct zgpio *gpio){
    char file[64];

    sprintf(file, SYSFS_DIRECTION, gpio->nr);
    return zfile_printfile(file, "out");
}

int zgpio_write(struct zgpio *gpio, int value){
    char file[64];

    sprintf(file, SYSFS_VALUE, gpio->nr);
    return zfile_printfile(file, "%d", value);
}

int zgpio_read(struct zgpio *gpio){
    char file[64], *str;
	int ret;

    sprintf(file, SYSFS_VALUE, gpio->nr);
    str = zfile_read_textfile(file);
    if (str == NULL) return -1;
    ret = atoi(str);
    g_free(str);
    return ret;
}

