/*
    zi2c.c - I2C library
    Copyright (C) 2018 Ladislav Vaiz <ok1zia@nagano.cz>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 2 as published by the Free Software Foundation.

*/

#include <libziaint.h>

#include <zi2c.h>

#include <fcntl.h>
#include <glib.h>
#ifdef Z_HAVE_LINUX_I2C_DEV_H
#include <linux/i2c-dev.h>
#endif
#include <stdint.h>
#ifdef Z_HAVE_SYS_IOCTL_H
#include <sys/ioctl.h>
#endif
#ifdef Z_HAVE_UNISTD_H
#include <unistd.h>
#endif


int zi2c_read(int busnr, int slave, unsigned int addr, size_t addrlen, void *buf, size_t readlen){
	int ret = -1;
#ifdef Z_HAVE_LINUX_I2C_DEV_H
    char filename[32], addrb[1];
    int fd;

    g_snprintf(filename, sizeof(filename), "/dev/i2c-%d", busnr);

    fd = open(filename, O_RDWR);
    if (fd < 0) return -1;
   
    ret = ioctl(fd, I2C_SLAVE, slave);
    if (ret < 0) {
        close(fd);
        return -2;
    }

    if (addrlen > 0){
        addrb[0] = addr;
        addrb[1] = addr / 256;
        addrb[2] = addr / 65536;
        addrb[3] = addr / 16777216;
        ret = write(fd, addrb, addrlen);
        if (ret < 0) {
            close(fd);
            return -3;
        }
    }

    if (readlen > 0){
        ret = read(fd, buf, readlen); 
        if (ret < 0) {
            close(fd);
            return -3;
        }
    }
    
    close(fd);
	ret = 0;
#endif
    return ret;
}

int zi2c_write_reg(int busnr, int slave, unsigned char addr, unsigned char value){
	int ret = -1;
#ifdef Z_HAVE_LINUX_I2C_DEV_H
    char filename[32], buf[2];
    int fd;

    g_snprintf(filename, sizeof(filename), "/dev/i2c-%d", busnr);

    fd = open(filename, O_RDWR);
    if (fd < 0) return -1;
   
    ret = ioctl(fd, I2C_SLAVE, slave);
    if (ret < 0) {
        close(fd);
        return -2;
    }

    buf[0] = addr;
    buf[1] = value;
    ret = write(fd, buf, 2);
    if (ret < 0) {
        close(fd);
        return -3;
    }
    
    close(fd);
    ret = 0;
#endif
	return ret;
}
