/*
    zsdltest.c - simple SDL test
    Copyright (C) 2018 Ladislav Vaiz <ok1zia@nagano.cz>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 2 as published by the Free Software Foundation.

*/

#include "libzia.h"

#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/kd.h>

int main(int argc, char *argv[]){

    int tty = open("/dev/tty1", O_RDWR);
    printf("opened tty1 as %d\n", tty);
    
    if(ioctl(tty, KDSETMODE, KD_GRAPHICS) == -1){
        fprintf(stderr, "Failed to set graphics mode on tty1\n");
    } 

    printf("Init\n");
    if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE)){
        fprintf(stderr, "SDL_Init failed\n");
        return(1);
    }
    printf("initialized\n");
	int flags = SDL_SWSURFACE|SDL_RESIZABLE|SDL_HWPALETTE/*|SDL_DOUBLEBUF*/;
    printf("SetVideoMode\n");
    SDL_Surface *screen = SDL_SetVideoMode(240, 320, 0, flags);
    printf("zsdl_init");
    struct zzsdl *zsdl = zsdl_init(screen, 16, 0);

    printf("makecol");
	int white = z_makecol(255, 255, 255);
	int gray = z_makecol(192, 192, 192);
	int red = z_makecol(255, 0, 0);
	int green = z_makecol(0, 255, 0);
	int blue = z_makecol(0, 0, 255);

    z_rect(screen, 0, 0, screen->w - 1, screen->h - 1, white); 
    z_rect(screen, 2, 2, screen->w - 3, screen->h - 3, white); 
	//z_line(screen, 0, 0, screen->w - 1, screen->h - 1, white);
	//z_line(screen, 0, screen->h - 1, screen->w - 1, 0, white);
	SDL_Rect r;
	r.x = 10; r.y = 10; r.w = 10; r.h = 10;
	SDL_FillRect(screen, &r, red);

	r.x = 20; r.y = 10; r.w = 10; r.h = 10;
	SDL_FillRect(screen, &r, green);

	r.x = 30; r.y = 10; r.w = 10; r.h = 10;
	SDL_FillRect(screen, &r, blue);
    
    char str[256];
    printf("VideoDriverName\n");
    SDL_VideoDriverName(str, sizeof(str)-1);
    str[sizeof(str)-1]='\0';
	int x = 10, y = 30;

	zsdl_printf(screen, x, y, white, 0, 0, "Driver: %s", str); y += zsdl->font_h + 6;
	zsdl_printf(screen, x, y, white, 0, 0, "%d x %d", screen->w, screen->h); y += zsdl->font_h + 6;
    printf("Flip\n");
	//SDL_Flip(screen);
    SDL_UpdateRect(screen, 0, 0, screen->w, screen->h);
    printf("sleep\n");
	sleep(2);

    int i;
    for (i = 0; i < 10; i++){
        usleep(500000);
        r.x = 0;
        r.y = 0;
        r.w = screen->w / 2;
        r.h = screen->h / 2;
        SDL_FillRect(screen, &r, red);
	    //SDL_Flip(screen);
        SDL_UpdateRect(screen, 0, 0, screen->w / 2, screen->h / 2);

        usleep(500000);
        r.x = 0;
        r.y = 0;
        r.w = screen->w / 2;
        r.h = screen->h / 2;
        SDL_FillRect(screen, &r, 0);
	    //SDL_Flip(screen);
        SDL_UpdateRect(screen, 0, 0, screen->w / 2, screen->h / 2);
    }  
    for (i = 0; i < 360; i++){
        char s[10];

        sprintf(s, "%03d", i);
        r.x = 10;
        r.y = y;
        r.w = zsdl->font_w * strlen(s);;
        r.h = zsdl->font_h;
    	zsdl_printf(screen, x, y, white, 0, 0, "%s", s);
        SDL_UpdateRect(screen, r.x, r.y, r.w, r.h);
        usleep(100000);
    }
    printf("Quit\n");
	SDL_Quit();

    
    return 0;
}
